/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: Container.java,v 1.3 2002/07/09 02:53:06 sreich Exp $
 */

package org.openejb.alt.assembler.classic.xml;

import java.util.Properties;
import org.openejb.OpenEJBException;
import org.openejb.alt.assembler.classic.ContainerInfo;
import org.w3c.dom.Node;

/**
 * A subclass of ContainerInfo filled with data from an XML file.
 * 
 * Populates the member variables of ContainerInfo in this classes initializeFromDOM method.
 * 
 * @author <a href="mailto:david.blevins@visi.com">David Blevins</a>
 * @author <a href="mailto:Richard@Monson-Haefel.com">Richard Monson-Haefel</a>
 * @see org.openejb.alt.assembler.classic.ContainerInfo
 * @see #Container.initializeFromDOM
 */
public class Container extends ContainerInfo {

    /**
     * Represents the <tt>description</tt> element in the XML config file.
     */
    public static final String DESCRIPTION = "description";

    /**
     * Represents the <tt>display-name</tt> element in the XML config file.
     */
    public static final String DISPLAY_NAME = "display-name";

    /**
     * Represents the <tt>container-name</tt> element in the XML config file.
     */
    public static final String CONTAINER_NAME = "container-name";
    
    /**
     * Represents the <tt>codebase</tt> element in the XML config file.
     */
    public static final String CODEBASE = "codebase";

    /**
        * Represents the <tt>codebase</tt> element in the XML config file.
     */
    public static final String CLASSNAME = "class-name";
        
    /** 
     * Parses out the values needed by this DomObject from the DOM Node passed in.
     */
    public static void initializeFromDOM(Node node, ContainerInfo container) throws OpenEJBException{
        container.description = DomTools.getChildElementPCData(node, DESCRIPTION);
        container.displayName = DomTools.getChildElementPCData(node, DISPLAY_NAME);
        container.containerName = DomTools.getChildElementPCData(node, CONTAINER_NAME);
        container.codebase = DomTools.getChildElementPCData(node, CODEBASE);
        container.className = DomTools.getChildElementPCData(node, CLASSNAME);
        container.properties = DomTools.readProperties(node);      
    }

}
