/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: BeanEntry.java,v 1.3 2002/08/21 00:28:10 dblevins Exp $
 */
package org.openejb.core.stateful;

import javax.ejb.SessionBean;
import javax.transaction.Transaction;

public class BeanEntry implements java.io.Serializable{
    protected final SessionBean bean;
    protected Object primaryKey;
    protected Object ancillaryState;   
    protected transient Transaction transaction; 
    protected long timeStamp;
    protected long timeOutInterval;
    protected boolean inQue = false;
        
    protected BeanEntry(SessionBean beanInstance, Object primKey, Object ancillary, long timeOut){
        bean = beanInstance;
        primaryKey = primKey;
        ancillaryState = ancillary;
        transaction = null;
        timeStamp = System.currentTimeMillis();
        timeOutInterval = timeOut;
    }
    protected boolean isTimedOut(){
        if(timeOutInterval == 0)
            return false;
        long now = System.currentTimeMillis();
        return (now - timeStamp)> timeOutInterval;
    }
    protected void resetTimeOut( ){
        if(timeOutInterval > 0) {
        timeStamp = System.currentTimeMillis();
    }
    }
}         
